/* vMac - Screen-X.c
 * X Window Interface
 * Copyright 1997 Michael Hanni (mhanni@sprintmail.com)
 */

/* This code is ment to be as portable as possible, it also should cover ALL X servers
 * If NOT let me know! I could add more compat things (especially for Sun Solaris stations 
 * and their warped OpenWindows, heh), but not until they are needed, for
 * I fear there are other things that are higher on the todo list! 
 */


#include "sysconfig.h"
#include "sysdeps.h"
#include "Screen.h"
#include "RAM.h"  
#include "Mouse.h"
#include "VIA.h"
#include "Keyboard.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <stdio.h>

#undef NOCURSOR

UBYTE vPage2 = 1;
UBYTE Draw_Delay = 2;  /* PC - Delay drawing of Screen every x time */
UBYTE Draw_Count = 0;  /* PC - Counter for Drawing Screen */

static Display 	*display;
static int 	screen;
static Window 	rootwin, mywin;
XImage *img1 = NULL;
XImage *img2 = NULL;
XColor black, white;
static GC 	blackgc;
static Visual 	*myvisual;
static int	mydepth, bit_unit;
static Colormap	mycmap;

static int 	macDisplayHeight = 342;
static int 	macDisplayWidth = 512;
int		mouse_x = 0, mouse_y = 0;
int 		buttonstate[3];
unsigned char	*image_Mem1, *image_Mem2;

static int	inwindow;
static Cursor 	xhairCursor, blankCursor;
static int 	cursorOn;

void Catch_X_Events();
int keycode2mac(XKeyEvent *event);

int Screen_Init(void)
{
	int			i, j;
	XTextProperty 		iconName, windowName;
    	XSetWindowAttributes 	wattr;
    	XPixmapFormatValues 	*xpfvs;
    	XWMHints 		win_hints;
    	XClassHint 		class_hints;
	XGCValues		gcv;

    	char 	*display_name = NULL;
    	char 	*window_name = "vMac";
    	char 	*icon_name = "vMac/UNIX";

	/* Get connection to X Server */

    	if ( (display=XOpenDisplay(display_name)) == NULL )
    	{
         	printf("Cannot connect to X server %s\n",
         	XDisplayName(display_name));
    	}
    	screen = DefaultScreen(display);

	mydepth  = DefaultDepth(display, screen);
  	myvisual = DefaultVisual(display, screen);
  	mycmap   = DefaultColormap(display, screen);

	XParseColor(display, mycmap, "#000000", &black);
		if (!XAllocColor(display, mycmap, &black))
		fprintf(stderr, "Whoops??\n");
	XParseColor(display, mycmap, "#ffffff", &white);
		if (!XAllocColor(display, mycmap, &white))
		fprintf(stderr, "Whoops??\n");
	                                
    	xpfvs = XListPixmapFormats(display, &i);
    	for (j = 0; j < i && xpfvs->depth != mydepth; j++, xpfvs++);
    	if (j == i) {
        	fprintf(stderr, "Your X server is feeling ill.\n");
        	return 0;
    	}
   	bit_unit = xpfvs->bits_per_pixel;
    	fprintf(stderr, "Using %d bit visual, %d bits per pixel\n", mydepth, bit_unit);

    	mywin = XCreateSimpleWindow(display, RootWindow(display, rootwin),
		0, 0, macDisplayWidth, macDisplayHeight, 4, 
		white.pixel,
		black.pixel);
	
	/* Set up some XString properties for window and icon names. */

    	if (XStringListToTextProperty(&window_name, 1, &windowName) ==0) {
      		printf("Um, this is an error.\n");
      		exit(-1);
    	}

    	if (XStringListToTextProperty(&icon_name, 1, &iconName) ==0) {
      		printf("This is an error as well.\n");
      		exit(-1);
    	}

	/* Hints, hints, hints */

    	win_hints.initial_state = NormalState;
    	win_hints.input = True;
    	win_hints.flags = StateHint | InputHint | WindowGroupHint;
    	XSetWMHints(display, mywin, &win_hints);

    	class_hints.res_name = "vmac";
    	class_hints.res_class = "vMac";
    	XSetClassHint(display, mywin, &class_hints);

    	XSetWMName(display, mywin, &windowName);
    	XSetWMIconName(display, mywin, &iconName);

	/* Is this overkill on XSelectInput? */

    	XSelectInput(display, mywin, ExposureMask | KeyPressMask | KeyReleaseMask |
      		ButtonPressMask | ButtonReleaseMask | StructureNotifyMask | PointerMotionMask |
      		EnterWindowMask | LeaveWindowMask );

    	XMapWindow(display, mywin);
    	XSync(display, 0);

    	blackgc = XCreateGC(display, mywin, 0, NULL);
	XSetState(display, blackgc, black.pixel, white.pixel, GXcopy, AllPlanes);

	buttonstate[0] = buttonstate[1] = buttonstate[2] = 0;
	
	blankCursor = XCreatePixmapCursor(display, XCreatePixmap(display, mywin, 1, 1, 1),
                                   	   XCreatePixmap(display, mywin, 1, 1, 1),
                                           &black, &white, 0, 0);
                                                                            
#ifdef NOCURSOR
	XDefineCursor(display, mywin, blankCursor);
	cursorOn = 1;
#endif

	image_Mem1 = (char *)malloc(macDisplayHeight * macDisplayWidth / 8);
    	img1 = XCreateImage(display, 
			myvisual, 
			1,
			XYBitmap, /* Apparently we ARE allowed to create 1-bit Bitmaps in a 16-bit XImage, damn docs :P */ 
			0,
			image_Mem1,
			macDisplayWidth,
			macDisplayHeight,
			32,
			512 / 8);
	img1->bitmap_bit_order = MSBFirst;
	img1->byte_order = MSBFirst;

	image_Mem2 = (char *)malloc(macDisplayHeight * macDisplayWidth / 8);
    	img2 = XCreateImage(display, 
			myvisual, 
			1,
			XYBitmap, 
			0, 
			image_Mem2,
			macDisplayWidth, 
			macDisplayHeight, 
			32, 
			512 / 8);
	img2->bitmap_bit_order = MSBFirst;
	img2->byte_order = MSBFirst;

    	return 0;

}

void Screen_Draw()
{
	char 	*src =(char *) RAM_Xlate(kMain_Buffer);
	char 	*src1 =(char *) RAM_Xlate(kAlternate_Buffer);
	char 	*dst =(char *) image_Mem1;
	char 	*dst1 =(char *) image_Mem2;
	int	i, j;

	// XFlushGC(display, blackgc);
	
	if ((Draw_Count % Draw_Delay) == 0)
	{
		if (vPage2 == 1) {
			for (i = 0; i < 512*342/8/2; i++)
			{
				*(dst++) = *(src+1);
				*(dst++) = *(src);
				src+=2;
			}
	       		XPutImage(display, mywin, blackgc, img1, 0, 0, 0, 0, img1->width, img1->height);
		}
		else {
			for (j = 0; j < 512*342/8/2; j++)
			{
				*(dst1++) = *(src1+1);
				*(dst1++) = *(src1);
				src1+=2;
			}
	       		XPutImage(display, mywin, blackgc, img2, 0, 0, 0, 0, img2->width, img2->height);
		}
	}
	Draw_Count++;

	Catch_X_Events();
}


void Screen_Reset()
{
	NULL;		/* need to write refresh, but first need to get Draw to work */
}


void Catch_X_Events()
{

   while(XPending(display) > 0)
   {
	XEvent event;  
	KeySym duh;
	long int duh2;
	XButtonEvent		*event_Button;
  	XPointerMovedEvent 	*event_Pointer;  
  	XEnterWindowEvent 	*event_Enter;
  	XLeaveWindowEvent 	*event_Leave;

        XNextEvent(display, &event);

	/*
	   NEW Aug 4: Mouse postion code in place, top left corner (0, 0), bottom right (512, 384).
	*/
                  
	switch(event.type) {
         	case KeyPress:
			duh = keycode2mac((XKeyEvent *)&event);
			//fprintf(stderr, "%d", XKeysymToKeycode(display, duh));
			duh2 = 12;
			Keyboard_Down(duh2);
			//fprintf(stderr, "KeyPress Event\n");
			break;
         	case KeyRelease:
			//duh = keycode2mac((XKeyEvent *)&event);
			//Keyboard_Up(XKeysymToString(duh));
			fprintf(stderr, "KeyRelease Event\n");
             		break;
         	case ButtonPress:
            		buttonstate[((XButtonEvent *)&event)->button-1] = 1; 
                     	break;
        	case ButtonRelease:
            		buttonstate[((XButtonEvent *)&event)->button-1] = 0;
			break;
         	case EnterNotify:
			mouse_x = ((XCrossingEvent *)&event)->x;
			mouse_y = ((XCrossingEvent *)&event)->y;
            		inwindow = 1;

			fprintf(stderr, "EnterNotify Event\n");
			break;
         	case LeaveNotify:
			inwindow = 0;

			fprintf(stderr, "LeaveNotify Event\n");
			break;
         	case FocusIn:
			fprintf(stderr, "FocusIn Event\n");
			break;
         	case FocusOut:   
			fprintf(stderr, "FocusOut Event\n");
			break;
         	case MotionNotify:
            		if (inwindow) {
                		mouse_x = ((XMotionEvent *)&event)->x;
                		mouse_y = ((XMotionEvent *)&event)->y;
            		}
			break;
         	case Expose:
			//fprintf(stderr, "Expose Event\n");
			break;
		default:
			/* This was getting annoying -Michael */
			// fprintf(stderr, "Ummm... Yeah, default\n");
			break;
	}

		//if ( mouse_y != -1 && mouse_x != -1 ) {
		// printf("Mouse Motion: %d, %d\n", mouse_x, mouse_y);
	Mouse_Update();
		//mouse_x = -1;
		//mouse_y = -1;
		//}

  }


}

int keycode2mac(XKeyEvent *event)
{
	KeySym ks;
    	int as = 0;

        ks = XLookupKeysym(event, as);

	return ks;
}

// VIA Interface Functions

UBYTE VIA_GORA6 (void) // Main/Alternate Screen Buffer
{
  #ifdef _VIA_Interface_Debug
  printf("VIA ORA6 attempts to be an input\n");
  #endif
  return 0;
}

void  VIA_PORA6 (UBYTE Data)
{
  vPage2 = Data;
}

UBYTE VIA_GORB6 (void) // Video Beam in Display
{
  return 0; // Assume it is
}

void  VIA_PORB6 (UBYTE Data)
{
  #ifdef _VIA_Interface_Debug
  printf("VIA ORB6 attempts to be an output\n");
  #endif
}
