/* vMac - SVGA-Linux.c
 * Linux SVGA Interface for vMac
 * Copyright 1997 Matt Gischer (matt@chaospro.com) and 
 * Phil Rutschman (machack@chaospro.com)
 * 
 */

#include "sysconfig.h"
#include "sysdeps.h"
#include "Screen.h"
#include "RAM.h"
#include "VIA.h"
#include "prefs.h"

#include <vga.h>
#include <vgakeyboard.h>
#include <vgamouse.h>

#include <stdio.h>

UBYTE vPage2 = 1;
UBYTE Draw_Count = 0;		/* PC - Counter for Drawing Screen */

volatile int buttonstate;
volatile int mouse_x, mouse_y;
static int macDisplayHeight = 342;
static int macDisplayWidth = 512;
int VIDMODE;
int KeyState[256];

void kb_handle(int ScanCode, int NewState);
void kb_handle2(int ScanCode, int NewState);

#define SC_ESC 0xFFFE
#define SC_CTR 0xFFFD
#define SC_ALT 0xFFFC
#define SC_SFT 0xFFFB
#define SC_CLK 0xFFFA

int MacKeys[128] =
{
    0xFFFF, SC_ESC, 0x1231, 0x1332, 0x1433, 0x1534, 0x1735, 0x1636,
    0x1A37, 0x1C38, 0x1939, 0x1D30, 0x1B2D, 0x183D, 0x3308, 0x3009,
    0x0C71, 0x0D77, 0x0E65, 0x0F72, 0x1174, 0x1079, 0x2075, 0x2269,
    0x1F6F, 0x2370, 0xFFFF, 0xFFFF, 0x490D, SC_CTR, 0x0061, 0x0173,
    0x0264, 0x0366, 0x0567, 0x0468, 0x266A, 0x286B, 0x256C, 0x293B,
    0x2727, 0xB27E, SC_SFT, 0x2A5C, 0x067A, 0x0778, 0x0863, 0x0976,
    0x0B62, 0x2D6E, 0x2E6D, 0x2B2C, 0x2F2E, 0x2C2F, SC_SFT, 0x9C2A,
    SC_ALT, 0x3120, SC_CLK, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF,
    0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0x1A37,
    0x1C38, 0x1939, 0x1B2D, 0x1534, 0x1735, 0x1636, 0x982B, 0x1231,
    0x1332, 0x1433, 0x1D30, 0x2F2E, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF,
    0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF,
    0x490D, SC_CTR, 0x2C2F, 0xFFFF, SC_ALT, 0xFFFF, 0xFFFF, 0x4D1E,
    0xFFFF, 0x461C, 0x421D, 0xFFFF, 0x481F, 0xFFFF, 0xFFFF, 0xFFFF,
    0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF,
    0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF
};

unsigned char EscapeKeys[128] =
{
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 96, 97, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 98, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 99,
    100, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 101, 102,
    103, 104, 0, 105, 0, 106, 0, 107,
    108, 109, 110, 111, 0, 0, 0, 0,
    0, 0, 0, 125, 126, 127, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0,
};

int Screen_Init(void)
{
    VIDMODE = 9;
    if (vga_setmode(VIDMODE) < 0) {
	sleep(1);
	vga_setmode(TEXT);
	fprintf(stderr, "SVGAlib doesn't like my video mode. Giving up.\n");
	return 0;
    }
    vga_setmousesupport(1);
    if (keyboard_init() != 0) {
	vga_setmode(TEXT);
	fprintf(stderr, "Are you sure you have a keyboard??\n");
	return 0;
    }
    keyboard_seteventhandler(kb_handle);
    if ((mouse_init("/dev/mouse", vga_getmousetype(), 10)) != 0) {
	printf("Mouse could not be initialized\n");
    }
    mouse_setxrange(64, 576);
    mouse_setyrange(69, 411);
    mouse_setposition(74, 80);
    printf("Drawing Every %i frames\n", thePrefs.FrameRate);
    return 0;
}

void Screen_Draw(void)
{
    int x, y;
    unsigned char *buf;
    ULONG cvtbuf[16];

    if (vPage2 == 1) {
	buf = (unsigned char *) RAM_Xlate(kMain_Buffer);
    } else {
	buf = (unsigned char *) RAM_Xlate(kAlternate_Buffer);
    }
    if ((Draw_Count % thePrefs.FrameRate) == 0) {
	for (y = 0; y < 342; y++, buf += 64) {
	    for (x = 0; x < 16; x++) {
/* Endianness fixing */
		cvtbuf[x] = (((255 - buf[x * 4 + 2]) * 256
			      + (255 - buf[x * 4 + 3])) * 256
			     + (255 - buf[x * 4])) * 256
		    + (255 - buf[x * 4 + 1]);
	    }
	    vga_drawscansegment((unsigned char *) cvtbuf, 64, 69 + y, 64);
	}
    }
    Draw_Count++;
    Catch_SVGA_Events();
}


void Screen_Reset()
{
    NULL;			/* need to write refresh, but first need to get Draw to work */
}


int Catch_SVGA_Events()
{
    int x, y;
    if (mouse_update()) {
	mouse_x = mouse_getx() - 64;
	mouse_y = mouse_gety() - 69;
	buttonstate = mouse_getbutton();
	Mouse_Update();
	keyboard_update();
    }
    return 0;
}

void kb_handle(int ScanCode, int NewState)
{
    static int IsEscape = 0, CapsLockCanChange = 1;
    int MacKey, Mods;

/* Stole all this directly out of the dos port.. thanks Gustavo! */
    if (ScanCode == 0xe0) {
	IsEscape = 1;
	return;
    }
    /* PAUSE Key */
    if (IsEscape > 1) {
	IsEscape++;
	if (IsEscape == 7) {
	    IsEscape = 0;
	}
	return;
    }
    if (ScanCode == 0xe1) {
	IsEscape = 2;
	return;
    }
    NewState = !(ScanCode & 0x80);
    ScanCode = ScanCode & 0x7f;
    if (IsEscape) {
	ScanCode = EscapeKeys[ScanCode];
	IsEscape = 0;
    }
    MacKey = MacKeys[ScanCode];

    if (MacKey == 0xFFFF)
	return;

    if (MacKey == SC_CLK) {
	if (NewState) {
	    if (CapsLockCanChange) {
		NewState = !KeyState[ScanCode];
		CapsLockCanChange = 0;
	    } else
		return;
	} else {
	    CapsLockCanChange = 1;
	    return;
	}
    }
    if (NewState && KeyState[ScanCode])
	NewState = 2;
    KeyState[ScanCode] = NewState;

    if (MacKey < 0xFF00) {
	Mods = 0;
	if (KeyState[56] || KeyState[100])
	    Mods = Mods + 0x0100;
	if (KeyState[42] || KeyState[54])
	    Mods = Mods + 0x0200;
	if (KeyState[58])
	    Mods = Mods + 0x0400;
	if (KeyState[29] || KeyState[97])
	    Mods = Mods + 0x0800;
	switch (NewState) {
	case 0:
	    PC_PostEvent(4, MacKey, Mods);
	    break;
	case 1:
	    PC_PostEvent(3, MacKey, Mods);
	    break;
	case 2:
	    PC_PostEvent(5, MacKey, Mods);
	    break;
	}
    }
}

// VIA Interface Functions

UBYTE VIA_GORA6(void)		// Main/Alternate Screen Buffer
 {
#ifdef _VIA_Interface_Debug
    printf("VIA ORA6 attempts to be an input\n");
#endif
    return 0;
}

void VIA_PORA6(UBYTE Data)
{
    vPage2 = Data;
}

UBYTE VIA_GORB6(void)		// Video Beam in Display
 {
    return 0;			// Assume it is

}

void VIA_PORB6(UBYTE Data)
{
#ifdef _VIA_Interface_Debug
    printf("VIA ORB6 attempts to be an output\n");
#endif
}
