// Cross Platform Compatible Preferences Code for vMac
// (C) 1997 Richard Bannister

// Based on code from Frodo - (C) 1994-1997 Christian Bauer

#include "sysconfig.h"
#include "sysdeps.h"
#include "prefs.h"

#ifdef _MacOS
#include "MacOS.h"
#endif

Prefs thePrefs;

void PrefsLoad(char *filename)
{
	FILE *file;
	char line[256], keyword[256], value[256];

	// Initialise the prefs with valid values
	#ifdef _MacOS
	thePrefs.BlackBackground=false;
	#endif
	thePrefs.FrameRate=1;
	strcpy(thePrefs.DrivePath[0], "Disk1.dsk");
	strcpy(thePrefs.DrivePath[1], "Disk2.dsk");
	strcpy(thePrefs.ROMPath[1],"vMac.ROM");
	thePrefs.CurrentROMPath=1;
	thePrefs.RAMSize=1;

	if ((file = fopen(filename, "r")) != NULL)
	{
		while(fgets(line, 255, file))
		{
			if (sscanf(line, "%s = %s\n", keyword, value) == 2)
			{
				if (!strcmp(keyword, "FrameRate"))
				{
					if (!strcmp(value, "1"))
						thePrefs.FrameRate = 1;
					else if (!strcmp(value, "2"))
						thePrefs.FrameRate = 2;
					else if (!strcmp(value, "3"))
						thePrefs.FrameRate = 3;
					else if (!strcmp(value, "4"))
						thePrefs.FrameRate = 4;
					else if (!strcmp(value, "5"))
						thePrefs.FrameRate = 5;
					else if (!strcmp(value, "6"))
						thePrefs.FrameRate = 6;
					else if (!strcmp(value, "7"))
						thePrefs.FrameRate = 7;
					else if (!strcmp(value, "8"))
						thePrefs.FrameRate = 8;
					else if (!strcmp(value, "9"))
						thePrefs.FrameRate = 9;
					else if (!strcmp(value, "10"))
						thePrefs.FrameRate = 10;
				}
				else if (!strcmp(keyword, "RAMSize"))
				{
					if (!strcmp(value, "1"))
						thePrefs.RAMSize = 1;
					else if (!strcmp(value, "2"))
						thePrefs.RAMSize = 2;
					else if (!strcmp(value, "4"))
						thePrefs.RAMSize = 4;
				} 
				#ifdef _MacOS
				else if (!strcmp(keyword, "BlackBackground"))
					thePrefs.BlackBackground = !strcmp(value, "TRUE");
				#endif
				#ifndef _MacOS // MacOS Port doesn't support paths yet
				else if (!strcmp(keyword, "DrivePath1"))
					strcpy(thePrefs.DrivePath[0], value);
				else if (!strcmp(keyword, "DrivePath2"))
					strcpy(thePrefs.DrivePath[1], value);
				else if (!strcmp(keyword, "ROMPath1"))
					strcpy(thePrefs.ROMPath[1],value);
				else if (!strcmp(keyword, "ROMPath2"))
					strcpy(thePrefs.ROMPath[2],value);
				else if (!strcmp(keyword, "ROMPath3"))
					strcpy(thePrefs.ROMPath[3],value);
				else if (!strcmp(keyword, "CurrentROMPath"))
				{
					if (!strcmp(value, "1"))
						thePrefs.CurrentROMPath = 1;
					else if (!strcmp(value, "2"))
						thePrefs.CurrentROMPath = 2;
					else if (!strcmp(value, "3"))
						thePrefs.CurrentROMPath = 3;
				}
				#endif
			}
		}
		fclose(file);
	}
}

void PrefsSave(char *filename)
{
	FILE *file;

	if ((file = fopen(filename, "w")) != NULL) {
		fprintf(file, "// vMac Configuration File\n");
		fprintf(file, "//\n");
		
		// MacOS Port; All options here will be configurable from the vMac application
		// If you want to add these lines in your port feel free - Richard Bannister
		#ifdef _MacOS
			fprintf(file, "// These options can all be changed from within the vMac\n");
			fprintf(file, "// application - there is no need to edit them here\n");
		#endif
		
		fprintf(file, "\n");
		
		fprintf(file, "// How many frames should be displayed? vMac by default\n");
		fprintf(file, "//draws every frame (1) but on slower machines the\n");
		fprintf(file, "//emulation speed can be adjusted by entering a higher\n");
		fprintf(file, "// figure here. (Minimum value = 1, Maximum Value = 10).\n\n");
		fprintf(file, "FrameRate = %d\n\n", thePrefs.FrameRate);

		#ifdef _MacOS
		fprintf(file, "//Do you want a black background covering the screen\n");
		fprintf(file, "// or do you want the emulator to run in a window?\n\n");
		fprintf(file, "BlackBackground = %s\n\n", thePrefs.BlackBackground ? "TRUE" : "FALSE");
		#endif
		
		fprintf(file, "// How much RAM do you want the emulated Mac Plus to\n");
		fprintf(file, "// have? (1 = 1 Mb, 2 = 2Mb, 4 = 4Mb).\n\n");
		fprintf(file, "RAMSize = %d\n", thePrefs.RAMSize);

		#ifndef _MacOS // MacOS Port doesn't support drive paths
		fprintf(file, "// Which disks are in the emulated disk drives? Where\n");
		fprintf(file, "// are they located?\n\n");
		fprintf(file, "DrivePath1 = %s\n", thePrefs.DrivePath[0]);
		fprintf(file, "DrivePath2 = %s\n\n", thePrefs.DrivePath[1]);
		
		fprintf(file, "// And where is the vMac.ROM file? If you have more\n");
		fprintf(file, "// than one you can put in the path for them all and\n");
		fprintf(file, "// vMac will allow you to switch between them!\n\n");
		fprintf(file, "ROMPath1 = %s\n", thePrefs.ROMPath[1]);
		fprintf(file, "ROMPath2 = %s\n", thePrefs.ROMPath[2]);
		fprintf(file, "ROMPath3 = %s\n", thePrefs.ROMPath[3]);
		fprintf(file, "CurrentROMPath = %d\n", thePrefs.CurrentROMPath);
		#endif
		
		fprintf(file, "\n");
		fprintf(file, "// End of File");
		fclose(file);
	}
}
